/*
 * gcc -o collatz -I/opt/local/include collatz.c -L/opt/local/lib -lgmp
 *
 * echo "11 27" | ./collatz
 * echo 63728127 | ./collatz
 * echo 123456789077 | ./collatz
 * echo 73786976294838206464 | ./collatz
 */

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <inttypes.h>
#include <errno.h>
#include <gmp.h>
#include <assert.h>

#define _POSIX_C_SOURCE 200809L

static uint64_t f_uint64(uint64_t n)
{
    assert(n > 0);
    return ((n % 2) == 0) ? n / 2 : 3 * n + 1;
}

static void f_mpz(mpz_t r, mpz_t n)
{
    mpz_t t;
    mpz_init(t);

    assert(mpz_cmp_ui(n, 0) == 1);
  
    mpz_mod_ui(t, n, 2);
    if (mpz_cmp_ui(t, 0) == 0) {
        mpz_div_ui(r, n, 2);
    } else {
        mpz_mul_ui(r, n, 3);
	mpz_add_ui(r, r, 1);
    }
    mpz_clear(t);
}

static void collatz_uint64(uint64_t n)
{
    if (n == 1) {
        puts("1");
    } else {
        (void) printf("%" PRIu64 " ", n);
	collatz_uint64(f_uint64(n));
    }
}

static void collatz_mpz(mpz_t n)
{
    mpz_t t;
    mpz_init(t);

    if (mpz_cmp_ui(n, 1) == 0) {
        puts("1");
    } else {
        (void) mpz_out_str(stdout, 10, n);
	printf(" ");
	f_mpz(t, n);
	collatz_mpz(t);
	mpz_clear(t);
    }
}

int main(int argc, char *argv[])
{
    char line[1024];
    const char *delim = " \t\n";
    char *word;

    uint64_t num;
    mpz_t number;
    mpz_init(number);

    while (fgets(line, sizeof(line), stdin)) {
	for (word = strtok(line, delim); word; word = strtok(NULL, delim)) {
	    errno = 0;
	    num = strtoull(word, NULL, 10);
	    if (errno == 0) {
		collatz_uint64(num);
	    } else if (mpz_set_str(number, word, 10) == 0) {
		collatz_mpz(number);
	    }
	}
    }
    mpz_clear(number);

    return 0;
}
